# FigurateNum

FigurateNum is a set of **figurate number generators** based on the book [Figurate Numbers (2012)](https://books.google.com.pe/books/about/Figurate_Numbers.html?id=cDxYdstLPz4C&redir_esc=y) by Michel Deza and Elena Deza.

FigurateNum generates the following types of **infinite sequences**:

- [x] 20 sequences of plane figurate numbers
- [x] 38 sequences of space figurate numbers
- [x] 18 sequences of multidimensional figurate numbers

## What is the purpose of FigurateNum?

FigurateNum facilitates the discovery of new patterns among sequences and enables various numerical calculations in mathematical projects and related endeavors. It can be integrated with other software to visualize the geometric objects described. Moreover, it serves as a valuable companion to the book!

## How to install?

```py
pip install figuratenum
```
* It is advisable to use a virtual environment.

## How to import figuratenum?

```py
import figuratenum as fgn
```

## How to use?

```py
>>> seq = fgn.hyperdodecahedral_numbers()

>>> first = next(seq)
>>> second = next(seq)
>>> third = next(seq)
>>> fourth = next(seq)

>>> print(first, second, third, fourth)
1 600 4983 19468
```

You could get a list of numbers using a loop:

```py
>>> generator = fgn.k_dimensional_centered_hypertetrahedron_numbers(21)
>>> sequence = []
>>> for _ in range(1, 15):
>>>     next_num = next(generator)
>>>     sequence.append(next_num)

>>> print(sequence)
[1, 23, 276, 2300, 14950, 80730, 376740, 1560780, 5852925, 20160075, 64512240, 193536720, 548354040, 1476337800]
```

Equivalently, using the array module:

```py
import array as arr_num

>>> generator = fgn.aztec_diamond_numbers()
>>> sequence = arr_num.array('i')
>>> for _ in range(1, 15):
>>>     next_num = next(generator)
>>>     sequence.append(next_num)

>>> print(sequence)
array('i', [4, 12, 24, 40, 60, 84, 112, 144, 180, 220, 264, 312, 364, 420])
```

## Plane figurate numbers

1. `polygonal_numbers`
2. `triangular_numbers`
3. `triangular_numbers`
4. `square_numbers`
5. `pentagonal_numbers`
6. `hexagonal_numbers`
7. `heptagonal_numbers`
8. `octagonal_numbers`
9. `nonagonal_numbers`
10. `decagonal_numbers`
10. `hendecagonal_numbers`
11. `dodecagonal_numbers`
12. `tridecagonal_numbers`
13. `tetradecagonal_numbers`
14. `pentadecagonal_numbers`
15. `hexadecagonal_numbers`
16. `heptadecagonal_numbers`
17. `octadecagonal_numbers`
18. `nonadecagonal_numbers`
19. `icosagonal_numbers`
20. `icosihenagonal_numbers`
21. `icosidigonal_numbers`
22. `icositrigonal_numbers`
23. `icositetragonal_numbers`
24. `icosipentagonal_numbers`
25. `icosihexagonal_numbers`
26. `icosiheptagonal_numbers`
27. `icosioctagonal_numbers`
28. `icosinonagonal_numbers`
29. `triacontagonal_numbers`
30. `centered_triangular_numbers`
31. `centered_square_numbers` = `diamond numbers`
32. `centered_pentagonal_numbers`
33. `centered_hexagonal_numbers`
34. `centered_heptagonal_numbers`
35. `centered_octagonal_numbers`
36. `centered_nonagonal_numbers`
37. `centered_decagonal_numbers`
38. `centered_hendecagonal_numbers`
39. `centered_dodecagonal_numbers` = `star_numbers`
40. `centered_tridecagonal_numbers`
41. `centered_tetradecagonal_numbers`
42. `centered_pentadecagonal_numbers`
43. `centered_hexadecagonal_numbers`
44. `centered_heptadecagonal_numbers`
45. `centered_octadecagonal_numbers`
46. `centered_nonadecagonal_numbers`
47. `centered_icosagonal_numbers`
48. `centered_icosihenagonal_numbers`
49. `centered_icosidigonal_numbers`
50. `centered_icositrigonal_numbers`
51. `centered_icositetragonal_numbers`
52. `centered_icosipentagonal_numbers`
53. `centered_icosihexagonal_numbers`
54. `centered_icosiheptagonal_numbers`
55. `centered_icosioctagonal_numbers`
56. `centered_icosinonagonal_numbers`
57. `centered_triacontagonal_numbers`
58. `centered_mgonal_numbers`
59. `pronic_numbers` = `heteromecic_numbers = oblong_numbers`
60. `polite_numbers`
61. `impolite_numbers`
62. `cross_numbers`
63. `aztec_diamond_numbers`
64. `polygram_numbers(m)` = `centered_star_polygonal_numbers(m)`
65. `pentagram_numbers`
66. `gnomic_numbers`
67. `truncated_triangular_numbers`
68. `truncated_square_numbers`
69. `truncated_pronic_numbers`
70. `truncated_centered_pol_numbers(m)` = `truncated_centered_pol_numbers(m)`
71. `truncated_centered_hexagonal_numbers` = `truncated_hex_numbers`
72. `generalized_mgonal_numbers(m, start_numb)`
73. `generalized_centered_pol_numbers(m, start_numb)`
74. `generalized_pronic_numbers(start_numb)`

## Space figurate numbers

1. `m_pyramidal_numbers(m)`
2. `cubic_numbers`
3. `tetrahedral_numbers`
4. `octahedral_numbers`
5. `dodecahedral_numbers`
6. `icosahedral_numbers`
7. `truncated_tetrahedral_numbers`
8. `truncated_cubic_numbers`
9. `truncated_octahedral_numbers`
10. `stella_octangula_numbers`
11. `centered_cube_numbers`
12. `rhombic_dodecahedral_numbers`
13. `hauy_rhombic_dodecahedral_numbers`
14. `centered_tetrahedron_numbers` = `centered_tetrahedral_numbers`
15. `centered_square_pyramid_numbers` = `centered_pyramid_numbers`
16. `centered_mgonal_pyramid_numbers(m)`
17. `centered_octahedron_numbers`
18. `centered_icosahedron_numbers` = `centered_cuboctahedron_numbers`
19. `centered_dodecahedron_numbers`
20. `centered_truncated_tetrahedron_numbers`
21. `centered_truncated_cube_numbers`
22. `centered_truncated_octahedron_numbers`
23. `centered_mgonal_pyramidal_numbers(m)`
24. `centered_hexagonal_pyramidal_numbers` = `hex_pyramidal_numbers`
25. `hexagonal_prism_numbers`
26. `mgonal_prism_numbers(m)`
27. `generalized_mgonal_pyramidal_numbers(m, start_num)`
28. `generalized_cubic_numbers(start_num)`
29. `generalized_octahedral_numbers(start_num)`
30. `generalized_icosahedral_numbers(start_num)`
31. `generalized_dodecahedral_numbers(start_num)`
32. `generalized_centered_cube_numbers(start_num)`
33. `generalized_centered_tetrahedron_numbers(start_num)`
34. `generalized_centered_square_pyramid_numbers(start_num)`
35. `generalized_rhombic_dodecahedral_numbers(start_num)`
36. `generalized_centered_mgonal_pyramidal_numbers(m, start_num)`
37. `generalized_mgonal_prism_numbers(m, start_num)`
38. `generalized_hexagonal_prism_numbers(start_num)`

## Multidimensional figurate numbers

1. `pentatope_numbers` = `hypertetrahedral_numbers` = `triangulotriangular_numbers`
2. `k_dimensional_hypertetrahedron_numbers(k)` = `k_hypertetrahedron_numbers(k)` = `regular_k_polytopic_numbers(k)` = `figurate_numbers_of_order_k(k)`
3. `biquadratic_numbers`
4. `k_dimensional_hypercube_numbers(k)` = `k_hypercube_numbers(k)`
5. `hyperoctahedral_numbers` = `hexadecachoron_numbers` = `four_cross_polytope_numbers` = `four_orthoplex_numbers`
6. `hypericosahedral_numbers` = `tetraplex_numbers` = `polytetrahedron_numbers` = `hexacosichoron_numbers`
7. `hyperdodecahedral_numbers` = `hecatonicosachoron_numbers` = `dodecaplex_numbers` = `polydodecahedron_numbers`
8. `polyoctahedral_numbers` = `icositetrachoron_numbers` = `octaplex_numbers` = `hyperdiamond_numbers`
9. `k_dimensional_hyperoctahedron_numbers(k)` = `k_cross_polytope_numbers(k)`
10. `four_dimensional_mgonal_pyramidal_numbers(m)` = `mgonal_pyramidal_numbers_of_the_second_order`
11. `k_dimensional_mgonal_pyramidal_numbers(k, m)` = `mgonal_pyramidal_numbers_of_the_k_2_th_order(k, m)`
12. `centered_biquadratic_numbers`
13. `k_dimensional_centered_hypercube_numbers(k)`
14. `centered_polytope_numbers`
15. `k_dimensional_centered_hypertetrahedron_numbers(k)`
16. `centered_hyperoctahedral_numbers` = `orthoplex_numbers`
17. `nexus_numbers`
18. `k_dimensional_centered_hyperoctahedron_numbers(k)`

## Contributing

FigurateNumber is currently under development, and we warmly invite your contributions. Just **fork** the project and then submit a **pull request**:

- Sequences from Chapters 1, 2, and 3 of the book
- New sequences not included in the book: If you have new sequences, please provide the source.
- Tests, documentation and errata in the book

When making commits, please use the following conventional prefixes to indicate the nature of the changes: `feat`, `refactor`, `fix`, `docs`, and `test`.

## License

[MIT License](https://github.com/edelveart/figuratenum/blob/main/LICENSE)


